// A quick library to get the Adafruit motor shield to run steppers on the Arduino Due
// Send any questions to Constantino: constantino@photocs.net
// http://www.photocs.net

#include "Arduino.h"
#include "Stepper.h"

static byte phase = 0;
static long lastStepTime = 0;
static byte motorNum = 0;


void Stepper::enable(void) { // lets on the H-bridges; reset the shift register, enable it and set the proper pins on the Arduino
	pinMode(MOTORLATCH, OUTPUT);
	pinMode(MOTORENABLE, OUTPUT);
	pinMode(MOTORDATA, OUTPUT);
	pinMode(MOTORCLK, OUTPUT);

	digitalWrite(MOTORENABLE, LOW);

	// enable both H bridges for motor ports 3/4
	pinMode(11, OUTPUT);
	pinMode(3, OUTPUT);
	digitalWrite(11, HIGH);
	digitalWrite(3, HIGH);  

	// enable both H bridges for motor ports 1/2
	pinMode(5, OUTPUT);
	pinMode(6, OUTPUT);
	digitalWrite(5, HIGH);
	digitalWrite(6, HIGH);
}


Stepper::Stepper(int motor) { // let's declare on which H-bridge our stepper is connected
	motorNum = motor;
	latch_tx(0, 0);  // "reset"
}

void Stepper::latch_tx(unsigned char coil, unsigned char motor) { // responsible for sending the proper configuration to the shift register.  Right now we are stepping in 'interleave' mode.
    
  byte i;
  byte on1 = 0;
  byte on2 = 0;
  
  
  
  // coil sequence goes as (for motor 1)
  // PHASE M1A M1B M2A M2B
  //  0  |  1   0   1   0
  //  1  |  1   0   0   0
  //  2  |  1   0   0   1
  //  3  |  0   0   0   1
  //  4  |  0   1   0   1
  //  5  |  0   1   0   0
  //  6  |  0   1   1   0
  //  7  |  0   0   1   0
  //  0  |  1   0   1   0
  
  digitalWrite(MOTORLATCH, LOW);
  digitalWrite(MOTORDATA, LOW);
  
  if (motor == 1) { // if we are on h-bridge 1
    switch(coil) {
      case 0:
        on1 = 5;
        on2 = 6;
        break;
      case 1:
        on1 = on2 = 5;
        break;
      case 2:
        on1 = 5;
        on2 = 3;
        break;
      case 3:
        on1 = on2 = 3;
        break;
      case 4:
        on1 = 4;
        on2 = 3;
        break;
      case 5:
        on1 = on2 = 4;
        break;
      case 6:
        on1 = 4;
        on2 = 6;
        break;
      case 7:  
        on1 = on2 = 6;
        break;
    }
  } else if (motor == 2) {
    switch(coil) {
      case 0:
        on1 = 7;
        on2 = 2;
        break;
      case 1:
        on1 = on2 = 7;
        break;
      case 2:
        on1 = 7;
        on2 = 0;
        break;
      case 3:
        on1 = on2 = 0;
        break;
      case 4:
        on1 = 0;
        on2 = 1;
        break;
      case 5:
        on1 = on2 = 1;
        break;
      case 6:
        on1 = 1;
        on2 = 2;
        break;
      case 7:  
        on1 = on2 = 2;
        break;
    }
  } else if (motor == 0) {
    on1 = on2 = 9; 
  }
  for (i = 0; i < 8; i++) {
    digitalWrite(MOTORCLK, LOW);
    if (i == on1 || i == on2) {
      digitalWrite(MOTORDATA, HIGH);
    } else {
      digitalWrite(MOTORDATA, LOW);      
    }
    digitalWrite(MOTORCLK, HIGH);
  }
  digitalWrite(MOTORLATCH, HIGH); 
}

void Stepper::stepOne(bool dir, long timeDelay ) { // responsible for stepping one single step
  unsigned long time = micros();

  
  if (time > lastStepTime + timeDelay) {
    latch_tx(phase, motorNum);
    if (dir) { // if we are going forward
		phase++;
		if (phase == 8) {
			phase = 0; // let's loop back to the first phase
		}
		} else { // if we are going reverse
		phase--;
		if (phase == 255) { // overflow since we can't have a negative number
			  phase = 7; // let's loop back to the last phase
		}
    }
    lastStepTime = time;
  }
}
